/*
  Name: conversionImageImageIpl.cpp
  Author: Marc Chaumont
  Date: 11/06/05 17:08
  Description: Permet de faire de convertion entre image et imageIpl
*/

#include <iostream>
#include <stdlib.h>
using namespace std;

#include "conversionImageImageIpl.hpp"

void ConversionImageImageIpl::convertImageToIplImage(const Image<double>& image, IplImage** imageIpl) {

  int x,y,i;
  int val;
  int width   = image.NbCol();
  int widthCV = (int)(floor(width/(double)2)*2); // = widthStep
  int height  = image.NbRow();
  
  //TAILLE des images pour openCV ...
  CvSize size;
  size.width  = width;
  size.height = height; 

  //ALLOCATION DES STRUCTURES
  (*imageIpl) = cvCreateImage(size, IPL_DEPTH_8U, 1);
  
  //CONVERSION EN UNSIGNED CHAR ET DANS UN TAMPON A LA BONNE TAILLE
  for (y=0,i=0; y<height; y++) {
    for (x=0; x<width; x++,i++) {
      val              = (int)rint(image(y,x));
      if (val<0)   val = 0;
      if (val>255) val = 255;
      ((*imageIpl)->imageData)[i] = (uchar)val;
    }
    i+= widthCV - width; //Les images IplImage sont stocks de sorte que la taille d'une ligne soit paire ...
  } 
}

void ConversionImageImageIpl::convertImageToIplImage(const Image<RGB>& image, IplImage** imageIpl) {

  int x,y,i;
  int width   = image.NbCol();
  int widthCV = 3*(int)(floor(width/(double)2)*2); // = widthStep
  int height  = image.NbRow();
  
  //TAILLE des images pour openCV ...
  CvSize size;
  size.width  = width;
  size.height = height; 

  //ALLOCATION DES STRUCTURES
  (*imageIpl) = cvCreateImage(size, IPL_DEPTH_8U, 3);
  //imageIpl->dataOrder == IPL_DATA_ORDER_PIXEL ...
  
  //CONVERSION EN UNSIGNED CHAR ET DANS UN TAMPON A LA BONNE TAILLE
  for (y=0,i=0; y<height; y++) {
    for (x=0; x<width; x++,i++) {
      ((*imageIpl)->imageData)[i] = image(y,x).B;
      i++;
      ((*imageIpl)->imageData)[i] = image(y,x).G;
      i++;
      ((*imageIpl)->imageData)[i] = image(y,x).R;
    }
    i+= widthCV - 3*width; //Les images IplImage sont stocks de sorte que la taille d'une ligne soit paire ...
  } 
}        

void ConversionImageImageIpl::convertIplImageToImage(const IplImage* imageIpl, Image<double>& image) {

  int x,y,i;
  
#ifdef RELEASE
  if ((*imageIpl).origin != IPL_ORIGIN_TL) {
    cout<<"ConvertIplImageToImage: Attention, le buffer stockant l'image dans imageIpl est invers...\n";
    exit(-1);
  }
//  if ((*imageIpl).depth != IPL_DEPTH_8U) {
//    cout<<"ConvertIplImageToImage: Erreur, fonction crit uniquement pour la conversion d'image stocke sur des unsigned char (IPL_DEPTH_8U)...\n";
//    exit(-1);      
//  }
  if ((*imageIpl).nChannels != 1) {
    cout<<"ConvertIplImageToImage: Erreur, fonction crit uniquement pour la conversion d'image en niveau de gris...\n";
    exit(-1);      
  }    
#endif
  
  //TAILLE des images IplImage (openCV)
  int width       = (*imageIpl).width;
  int height      = (*imageIpl).height; 
  int widthStep   = (*imageIpl).widthStep; //int widthCV = (int)(floor(width/(double)2)*2); 
    
  //RETAILLE DE L'IMAGE image
  image.Resize(height, width);
  
  //REMPLISSAGE DE LA STRUCTURE image
  for (y=0,i=0; y<height; y++) {
    for (x=0; x<width; x++,i++) {
      image(y,x) = ((*imageIpl).imageData)[i];
    }
    i+= widthStep - width; //Les images IplImage sont stocks de sorte que la taille d'une ligne soit de taille widthStep ...
  } 
}    

#ifdef NOTDEF
void ConversionImageImageIpl::convertIplImageToImage_ShallowSetBuff(const IplImage* imageIpl, Image<double>& image) {

#ifdef RELEASE
  if ((*imageIpl).origin != IPL_ORIGIN_TL) {
    cout<<"ConversionImageImageIpl::convertIplImageToImage_ShallowSetBuff: Attention, le buffer stockant l'image dans imageIpl est invers...\n";
    exit(-1);
  }
  if ((*imageIpl).nChannels != 1) {
    cout<<"ConversionImageImageIpl::convertIplImageToImage_ShallowSetBuff: Erreur, fonction crit uniquement pour la conversion d'image en niveau de gris...\n";
    exit(-1);      
  }
#endif

  //TAILLE des images IplImage (openCV)
  int width       = (*imageIpl).width;
  int height      = (*imageIpl).height; 
  int widthStep   = (*imageIpl).widthStep; //int widthCV = (int)(floor(width/(double)2)*2); 

  //On ne peut faire une copie par copie de pointeur que uniquement si la taille 
  //(widthStep) utilise pour le stockage d'une ligne de l'image Ipl est gale  
  //la largeur de l'image
  if (widthStep == width) { 
    //COPIE PAR POINTEUR
    image.ShallowSetBuff(imageIpl->imageData, height,width); //Juste un passage de pointeur
  }
  else {
    cout<<"ConversionImageImageIpl::convertIplImageToImage_ShallowSetBuff: On est oblig de faire une copie en profondeur !\n";
    //COPIE EN PROFONDEUR ...
    convertIplImageToImage(imageIpl, image);
  }    
}
#endif

void ConversionImageImageIpl::convertIplImageToImage(const IplImage* imageIpl, Image<RGB>& image) {

  int x,y,i;
  
#ifdef RELEASE
  if ((*imageIpl).origin != IPL_ORIGIN_TL) {
    cout<<"ConvertIplImageToImage: Attention, le buffer stockant l'image dans imageIpl est invers...\n";
    exit(-1);
  }
  if ((*imageIpl).nChannels != 3) {
    cout<<"ConvertIplImageToImage: Erreur, fonction crit uniquement pour la conversion  3 niveau composantes...\n";
    exit(-1);      
  }    
#endif
  
  //TAILLE des images IplImage (openCV)
  int width       = (*imageIpl).width;
  int height      = (*imageIpl).height; 
  int widthStep   = (*imageIpl).widthStep; //int widthCV = (int)(floor(width/(double)2)*2); 
  cout<<widthStep<<endl;
      
  //RETAILLE DE L'IMAGE image
  image.Resize(height, width);
  
  //REMPLISSAGE DE LA STRUCTURE image
  if (imageIpl->dataOrder == IPL_DATA_ORDER_PLANE) {
    cout<<"ConvertIplImageToImage: IPL_DATA_ORDER_PLANE n'est pas test\n";
    exit(-1);     
    int planSize    = height*widthStep;
    int twoPlanSize = 2*planSize;
    for (y=0,i=0; y<height; y++) {
      for (x=0; x<width; x++,i++) {
        image(y,x).B = ((*imageIpl).imageData)[i];
        image(y,x).G = ((*imageIpl).imageData)[i+planSize];
        image(y,x).R = ((*imageIpl).imageData)[i+twoPlanSize];
      }
      i+= widthStep - width; //Les images IplImage sont stocks de sorte que la taille d'une ligne soit de taille widthStep ...
    } 
  }    
  else { //imageIpl->dataOrder == IPL_DATA_ORDER_PIXEL
    for (y=0,i=0; y<height; y++) {
      for (x=0; x<width; x++,i++) {
        image(y,x).B = ((*imageIpl).imageData)[i];
        i++;
        image(y,x).G = ((*imageIpl).imageData)[i];
        i++;
        image(y,x).R = ((*imageIpl).imageData)[i];
      }
      i+= widthStep - 3*width; //Les images IplImage sont stocks de sorte que la taille d'une ligne soit de taille widthStep ...
    }       
  }    
}    

