#include <iostream>
#include <stdlib.h>

using namespace std;
//#include "math.hpp"

static double PARAM_a     = 0.055;
static double PARAM_gamma = 2.4;

static double fonction_g(double k) {
  if (k > 0.04045) {
    return pow( ((k+PARAM_a)/(1+PARAM_a)) , PARAM_gamma);
  }
  else {
    return k/12.92;
  }
}          
    
//static void RGBtoXYZ(double r, double g, double b, double& x, double& y, double& z) {
//  x = 0.412424*fonction_g(r/255) + 0.357579*fonction_g(g/255) + 0.180464*fonction_g(b/255);
//  y = 0.212656*fonction_g(r/255) + 0.715158*fonction_g(g/255) + 0.072186*fonction_g(b/255);
//  z = 0.019332*fonction_g(r/255) + 0.119193*fonction_g(g/255) + 0.950444*fonction_g(b/255);
//}
static void RGBtoXYZ(double r, double g, double b, double& x, double& y, double& z) {
  r = r/255.;
  g = g/255.;
  b = b/255.;
  x = 0.607*r + 0.174*g + 0.2*b;
  y = 0.299*r + 0.587*g + 0.114*b;
  z =           0.066*g + 1.116*b;
}


static double fonction_C(double c) {
  if (c <= 0.0031308)
    return rint(255*12.92*c);
  else
    return rint(255*( (1+PARAM_a)*pow(c, 1/PARAM_gamma) - PARAM_a));
}
   
//static void XYZtoRGB(double x, double y, double z, double& r, double& g, double& b) {
//  r =  3.2406*fonction_C(x)  -1.5372*fonction_C(y) -0.4986*fonction_C(z);
//  g = -0.9689*fonction_C(x)  +1.8758*fonction_C(y) +0.0415*fonction_C(z);
//  b =  0.0557*fonction_C(x)  -0.2040*fonction_C(y) +1.0570*fonction_C(z);
//} 
 
static void XYZtoRGB(double x, double y, double z, double& r, double& g, double& b) {
  r =  1.910457991  *x -0.5339398010 *y  -0.2878337463*z;
  g = -0.9844360194 *x +1.998503892  *y  -0.02772602136*z;
  b =  0.05821933448*x -0.1181910904 *y  +0.8976970586*z;
  r = r*255;
  g = g*255;
  b = b*255;
}

static double fonction_f(double t) {
  if (t>0.008856)
    return pow(t, 1/(double)3);
  else 
    return 7.787*t + 16/(double)116;
}    

//The three parameters in the model represent 
//  - the lightness of the color (L*, L*=0 yields black 
//        and L*=100 indicates white), 
//  - its position between magenta and green (a*, negative 
//        values indicate green while positive values indicate magenta) 
//  - and its position between yellow and blue (b*, negative values 
//        indicate blue and positive values indicate yellow).
// D65 	CIE 1964, Note : Television, sRGB color space K = 6500
const double PARAM_xn = 0.950456; 	
const double PARAM_yn = 1.;
const double PARAM_zn = 1.089058;

//"E" or "Equal Energy" white point :
//const double PARAM_xn = 1/(double)3; 	
//const double PARAM_yn = 1/(double)3;
//const double PARAM_zn = 1/(double)3;
    
    
//Cf livre Digital Image Processing Thirs Edition page 84    
static void XYZtoLab(double x, double y, double z, double& l, double& a, double& b) {
  l = 116*fonction_f(y/PARAM_yn) - 16;
  a = 500*( fonction_f(x/PARAM_xn) - fonction_f(y/PARAM_yn) );
  b = 200*( fonction_f(y/PARAM_yn) - fonction_f(z/PARAM_zn) );
}

static double fonction_fy(double l) {
  return (l + 16)/(double)116;
}    
static double fonction_fx(double fy, double a) {
  return fy + a/(double)500;
}    
static double fonction_fz(double fy, double b) {
  return fy - b/(double)200;
}    
static double PARAM_delta = 6/(double)29;
static void LabtoXYZ(double l, double a, double b, double& x, double& y, double& z) {
    
  double fx, fy, fz;
    
  if ((fy = fonction_fy(l)) > PARAM_delta) 
    y = PARAM_yn*fy*fy*fy;
  else
    y = (fy - 16/(double)116) * 3 *PARAM_delta * PARAM_delta * PARAM_yn;

  if ((fx = fonction_fx(fy, a)) > PARAM_delta) 
    x = PARAM_xn*fx*fx*fx;
  else
    x = (fx - 16/(double)116) * 3 * PARAM_delta * PARAM_delta * PARAM_xn;

  if ((fz = fonction_fz(fy, b)) > PARAM_delta) 
    z = PARAM_zn*fz*fz*fz;
  else
    z = (fz - 16/(double)116) * 3 * PARAM_delta * PARAM_delta * PARAM_zn;
}

static void RGBtoLab(double r, double g, double b, double& ll, double& aa, double& bb) {
  double x,y,z;
  RGBtoXYZ(r, g, b, x, y, z);
  XYZtoLab(x, y, z, ll, aa, bb);
}  

static void LabtoRGB(double ll, double aa, double bb, double& r, double& g, double& b) {
  double x,y,z;
  LabtoXYZ(ll, aa, bb, x, y, z);
  XYZtoRGB(x, y, z, r, g, b);
} 
