/*
  Name: main_embed.cpp
  Author: Marc CHAUMONT
  Date: 18/02/07 15:05
  Description: Permet d'appeler les routines d'insertion du livre de Cox-Miller-Bloom
               avec un appel + ou - conforme  l'appel pour le soft optimark
*/

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <vector>

using namespace std;

//Pour les images
#include "image.H"
#include "imageio.H"
#include "imagetools.H"
#include "ReversibleWatermarking.hpp"

#define MIN(x,y) ( ((x)<(y)) ? x: y) //macro maximum

/*VARIABLE GLOBALES*/
const int STR_LEN = 1000;
char imageOriginalName[STR_LEN]; //Nom du fichier image  marquer
char imageMarquedName[STR_LEN];  //Nom du fichier image marqu
//char message[STR_LEN];         //Nom du fichier message
//int messLength;                //Longueur du message en nombre de bits

int n=4;                         //entier suprieur ou gal  1 : cf. article Coltuc ICIP 2007

static void MakeRandomMessage( unsigned int seed, vector<int>& message, int messSize);

static int getRandomInt(int bSup) {
  if (bSup <= 0) return 0;
  else           return (int)rint(bSup*(rand()/(double)RAND_MAX));  
}

// Permet de re-arranger pseudo-aleatoirement un vecteur de coefficient
// On utilise ici une fonction peu sre (rand)
static void randomizeOrder(unsigned char* coeff, int size, unsigned int seed) {

  cout<<"project::randomizeOrder: size = "<<size<<", seed="<<seed<<endl;     
  int i, nb, nbelem;
  srand(seed);                                  //Initialisation de rand    
  vector<unsigned char> lst_coeff(size);               //Liste de coefficients   
  
  //RECOPIE DES COEFFICIENTS DANS LA LISTE
  // ET DUPLICATION DU TABLEAU COEFF
  for (i=0; i<size; i++) {         
    lst_coeff[i]  = coeff[i];                   //Recopie des coeff dans la liste
  }
      
  //REMPLISSAGE DE coeff
  nbelem = size;                                //Nombre d'lment de la liste
  for (nb=0; nb<size; nb++) {                   //Remplissage de nb coefficients
    i = getRandomInt(nbelem-1);                 //Tirage d'un nombre entier dans l'intervalle [0 , taille_de_la_liste - 1]  
#ifndef RELEASE
    if ((i<0) || (i>=nbelem)) {
      cout<<"project::randomizeOrder: Erreur : getRandomInt n'a pas tir un nombre appartenant au bon interval!!!\n";
      cout<<"   nombre tir : "<<i<<", interval : [0, "<<nbelem-1<<"]\n";
      exit(-1);
    }
#endif //RELEASE    
    coeff[nb]    = lst_coeff[i];                   //Recopie du coefficient i
    
    //SUPPRESSION DU COEFFICIENT DE LA LISTE
    lst_coeff[i] = lst_coeff[nbelem-1];         //Recopie du dernier lment de la liste dans la case i;
    lst_coeff.pop_back();                       //Diminution de 1 de la taille de la liste
    
    //ITERATION SUIVANTE
    nbelem--;
  }
}

static void randomizeOrder_inv(unsigned char* coeff, int size, unsigned int seed) {

  cout<<"project::randomizeOrder_inv: size = "<<size<<", seed="<<seed<<endl;     
  int i, nb, nbelem;
  srand(seed);                                        //Initialisation de rand    
  vector<int> lst_pointeur_coeff(size);               //Liste de coefficients   
  unsigned char* coeffOrderRandomized = new unsigned char[size];    //Copie des coefficients en entre
  
  //RECOPIE DES COEFFICIENTS DANS LA LISTE
  for (i=0; i<size; i++) {    
    lst_pointeur_coeff[i]    = i;                        //Recopie d'indice des coeffs dans la liste
    coeffOrderRandomized[i]  = coeff[i];                 //Copie des coefficients en entre
  }
      
  //REMPLISSAGE DE coeff
  nbelem = size;                                      //Nombre d'lment de la liste
  for (nb=0; nb<size; nb++) {                         //Remplissage de nb coefficients
    i = getRandomInt(nbelem-1);                       //Tirage d'un nombre entier dans l'intervalle [0 , taille_de_la_liste - 1]  
#ifndef RELEASE
    if ((i<0) || (i>=nbelem)) {
      cout<<"project::randomizeOrder_inv: Erreur : getRandomInt n'a pas tir un nombre appartenant au bon interval!!!\n";
      cout<<"   nombre tir : "<<i<<", interval : [0, "<<nbelem-1<<"]\n";
      exit(-1);
    }
#endif //RELEASE    

    coeff[lst_pointeur_coeff[i]] = coeffOrderRandomized[nb]; //Recopie du coefficient  la position lst_coeff[i]
    
    //SUPPRESSION DU COEFFICIENT DE LA LISTE
    lst_pointeur_coeff[i] = lst_pointeur_coeff[nbelem-1]; //Recopie du dernier lment de la liste dans la case i;
    lst_pointeur_coeff.pop_back();                        //Diminution de 1 de la taille de la liste

    //ITERATION SUIVANTE
    nbelem--;
  }
  delete[] coeffOrderRandomized;
}


//==============================================================================
void printUsage(char* name) {

  cout << "\nUsage : "
       <<name
       <<" inputImage.pgm outputImage.pgm\n";
  exit(-1);
}

//==============================================================================
void processOptionsInLine(int argc, char** argv){  
  if (argc != 3) {
    printUsage(argv[0]);
  }

  //RECOPIE DU NOM DE L'IMAGE DANS LA VARIABLE imageOriginalName
  strcpy(imageOriginalName, argv[1]);  
    
  //RECOPIE DU NOM DE L'IMAGE DANS LA VARIABLE imageMarquedName
  strcpy(imageMarquedName, argv[2]);  
}

//==============================================================================
//void processAttributes(){
//}

static void affichePattern(double* wr, int taille);
//==============================================================================
int main(int argc, char *argv[]) {

  char car;
  int i, choix, 
      m;   //Entier message bit 0 ou 1
  int* b;  //Vecteur binaire
  int* c;  //Vecteur binaire
  int* mc; //Vecteur binaire cod
  int lg;  //Longueur d'un vecteur binaire
  double scaling;
  
  processOptionsInLine(argc, argv);
  //processAttributes();

   
  //LECTURE DE L'IMAGE
  Image<octet> image;
  cout<<"Lecture de "<<imageOriginalName<<endl;
  ImageIO<octet>::ReadPGM(imageOriginalName, image);
  //unsigned int seed = 0;
  //randomizeOrder(image.PixelPointer(0), image.BufferSize(), seed);
  Image<octet> imageMarked(image); //Recopie de image dans image marque
  
  vector<int> message; // Suite de coefficients extrait compris dans [0, n-1]                                 
  int messSize = 500000;
  MakeRandomMessage( 220, message, messSize);
  
  //INSERTION  
  ReversibleWatermarking::embed3( image.PixelPointer (0),       //unsigned char *c, //image en entre
                                  image.NbRow(),                //int height,
                                  image.NbCol(),                //int width, 
                                  message,                      // Suite de coefficients extrait compris dans [0, n-1]
                                  imageMarked.PixelPointer (0), //Image marque
                                  n);                           //valeur suprieure ou gale  1 : cf. coltuc ICIP 2007

  //SAUVEGARDE DE L'IMAGE MARQUEE
  //randomizeOrder_inv(imageMarked, imageMarked.BufferSize(), seed);
  ImageIO<octet>::WritePGM(imageMarquedName, imageMarked);
  //exit(-1);


  //EXTRACTION
  int messLength;
  Image<octet> imageRebuild(image.NbRow(), image.NbCol());
  vector<int> messageExtract; // Suite de coefficients extrait compris dans [0, n-1]                                 
  ReversibleWatermarking::extract3( imageMarked.PixelPointer (0),   //unsigned char *c, //image en entre
                                    image.NbRow(),                  //int height,
                                    image.NbCol(),                  //int width, 
                                    messageExtract,                 // Suite de coefficients extrait compris dans [0, n-1]
                                    imageRebuild.PixelPointer (0),  //Image rebuild
                                    n);

  ImageIO<octet>::WritePGM("rebuild.pgm", imageRebuild);

  //IMAGE DE DIFFERENCE
  Image<octet> diff(image.NbRow(), image.NbCol());
  int val;
  for (i=0; i<diff.BufferSize(); i++) {
    //Si    val > 127 : modification suprieur   0
    //Sinon si val < 127 : modification infrieur   0
    val = (int)rint(127 + image[i]-imageMarked[i]);
    if (val<0)   val = 0;
    if (val>255) val = 255;
    diff[i] = (octet)val;
  }    
  ImageIO<octet>::WritePGM("diff.pgm", diff);

  //cout<<"messageExtract.size() = "<<messageExtract.size()<<endl;
  //if (message.size() !=  messageExtract.size()) {
  //cout<<"main: Erreur, le message extrait n'est pas de mme taille que celui insr\n";
  //exit(-1);                     
  //}
  int min = MIN(message.size() , messageExtract.size());
  cout<<"30 premires valeur du message:\n";
  for( i = 0; i < min; i++) {
    if (i< 30) cout<<message[i]<<" , ";
    if (message[i] !=  messageExtract[i]) {
      cout<<"main: Erreur, le message extrait n'est pas le mme que celui insr\n";
      exit(-1);
    }
  }
  cout<<endl;

  //system("PAUSE");	
  //return 0;
}
  
/* ranf() is uniform in 0..1 */
static double ranf() { 
  return (rand()/(double)RAND_MAX);
}  
    
// Suite de coefficients extrait compris dans [0, n-1]
static void MakeRandomMessage( unsigned int seed, vector<int>& message, int messSize) {
  int i;
  message.resize(0);
  srand(seed); //re-initialisaion de la semance
  for( i = 0; i < messSize; i++)
    message.push_back((int)rint(ranf()*(n-1)));

  //TEST de Scurit (POST-CONDITION)
  for( i = 0; i < messSize; i++) {
    if ((message[i] < 0) || (message[i] > n-1)) {
      cout<<"main::MakeRandomMessage: Erreur, message n'est pas dans le bon interval\n";
      exit(-1);
    }
  }
}
